package com.jiayue.ssi.filter;


import com.jiayue.ssi.servlet.XssKeywordsHttpServletRequestWrapper;
import com.jiayue.ssi.util.ResponseInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * xss和sql过滤器
 *
 * @author xsl
 * @since 2023/05/16
 */
@RequiredArgsConstructor
@Order(2)
@Slf4j
@Component
public class XssKeywordsFilter extends OncePerRequestFilter {
    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) {
        try {
            //跨域设置
//        if(servletResponse instanceof HttpServletResponse){
//            HttpServletResponse httpServletResponse=(HttpServletResponse)servletResponse;
//            //通过在响应 header 中设置 ‘*’ 来允许来自所有域的跨域请求访问。
//            httpServletResponse.setHeader("Access-Control-Allow-Origin", "*");
//            //通过对 Credentials 参数的设置，就可以保持跨域 Ajax 时的 Cookie
//            //设置了Allow-Credentials，Allow-Origin就不能为*,需要指明具体的url域
//            //httpServletResponse.setHeader("Access-Control-Allow-Credentials", "true");
//            //请求方式
//            httpServletResponse.setHeader("Access-Control-Allow-Methods", "*");
//            //(预检请求)的返回结果(即 Access-Control-Allow-Methods 和Access-Control-Allow-Headers 提供的信息) 可以被缓存多久
//            httpServletResponse.setHeader("Access-Control-Max-Age", "86400");
//            //首部字段用于预检请求的响应。其指明了实际请求中允许携带的首部字段
//            httpServletResponse.setHeader("Access-Control-Allow-Headers", "*");
//        }

            XssKeywordsHttpServletRequestWrapper xssRequest = new XssKeywordsHttpServletRequestWrapper(request);
//            String method = request.getMethod();
//
//            String param = "";

//            if ("POST".equalsIgnoreCase(method)) {
//                param = this.getBodyString(xssRequest.getReader());
//                if (StringUtils.isNotBlank(param)) {
//                    if (xssRequest.checkXSSAndSql(param)) {
//                        ResponseInfo.doResponse(response, "您所访问的页面请求中有违反安全规则元素存在，拒绝访问!", 410);
//                        return;
//                    }
//                }
//            }

            if (xssRequest.checkParameter()) {
                ResponseInfo.doResponse(response, "您所访问的页面请求中有违反安全规则元素存在，拒绝访问!", 410);
                return;
            }

            filterChain.doFilter(xssRequest, response);
        } catch (Exception e) {
//            log.error(IPUtils.getIpAddr(request) + "访问系统失败", e);
            ResponseInfo.doResponse(response, "访问失败,联系管理员！", 401);
            return;
        }

    }


    // 获取request请求body中参数
    public String getBodyString(BufferedReader br) {
        String inputLine;
        String str = "";
        try {
            while ((inputLine = br.readLine()) != null) {
                str += inputLine;
            }
            br.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return str;

    }


    @Override
    public void destroy() {

    }
}
